import os
import shutil
import tempfile
from datetime import datetime

import pytest
from lektor.types import Type


class DatetimeType(Type):
    def value_from_raw(self, raw):
        return datetime.strptime(raw.value, "%Y-%m-%d %H:%M:%S")


@pytest.fixture(scope="function")
def project(request):
    from lektor.project import Project

    return Project.from_path(os.path.join(os.path.dirname(__file__), "demo-project"))


@pytest.fixture(scope="function")
def env(request, project):
    from lektor.environment import Environment

    e = Environment(project)
    e.types["datetime"] = DatetimeType  # As if we had a datetime plugin.
    return e


@pytest.fixture(scope="function")
def pad(request, env):
    from lektor.db import Database

    return Database(env).new_pad()


def make_builder(request, pad):
    from lektor.builder import Builder

    out = tempfile.mkdtemp()
    b = Builder(pad, out)

    def cleanup():
        try:
            shutil.rmtree(out)
        except (OSError, IOError):
            pass

    request.addfinalizer(cleanup)
    return b


@pytest.fixture(scope="function")
def builder(request, pad):
    return make_builder(request, pad)


@pytest.fixture(scope="function")
def F():
    from lektor.db import F

    return F


@pytest.fixture(scope="function")
def reporter(request, env):
    from lektor.reporter import BufferReporter

    r = BufferReporter(env)
    r.push()
    request.addfinalizer(r.pop)
    return r



import os

from lxml import objectify


def test_typical_feed(pad, builder):
    failures = builder.build_all()
    assert not failures
    feed_path = os.path.join(builder.destination_path, "typical-blog/feed.xml")
    feed = objectify.parse(open(feed_path)).getroot()

    assert "Feed One" == feed.title
    assert "My Summary" == feed.subtitle
    assert "text" == feed.subtitle.attrib["type"]
    assert "A. Jesse Jiryu Davis" == feed.author.name
    assert "http://x.com/typical-blog/" == feed.link[0].attrib["href"]
    assert "http://x.com/typical-blog/feed.xml" == feed.link[1].attrib["href"]
    assert "self" == feed.link[1].attrib["rel"]
    assert "Lektor Atom Plugin" == feed.generator

    assert 2 == len(feed.entry)
    post2, post1 = feed.entry  # Most recent first.

    assert "Post 2" == post2.title
    assert "2015-12-13T00:00:00Z" == post2.updated
    assert "<p>bar</p>" == str(post2.content).strip()
    assert "html" == post2.content.attrib["type"]
    assert "http://x.com/typical-blog/post2/" == post2.link.attrib["href"]
    base = post2.attrib["{http://www.w3.org/XML/1998/namespace}base"]
    assert "http://x.com/typical-blog/post2/" == base
    assert "Armin Ronacher" == post2.author.name

    assert "Post 1" == post1.title
    assert "2015-12-12T00:00:00Z" == post1.updated
    assert "<p>foo</p>" == str(post1.content).strip()
    assert "html" == post1.content.attrib["type"]
    assert "http://x.com/typical-blog/post1/" == post1.link.attrib["href"]
    base = post1.attrib["{http://www.w3.org/XML/1998/namespace}base"]
    assert "http://x.com/typical-blog/post1/" == base
    assert "A. Jesse Jiryu Davis" == post1.author.name


def test_custom_feed(pad, builder):
    failures = builder.build_all()
    assert not failures
    feed_path = os.path.join(builder.destination_path, "custom-blog/atom.xml")
    feed = objectify.parse(open(feed_path)).getroot()

    assert "Feed Three" == feed.title
    assert "<p>My Description</p>" == str(feed.subtitle).strip()
    assert "html" == feed.subtitle.attrib["type"]
    assert "A. Jesse Jiryu Davis" == feed.author.name
    assert "http://x.com/custom-blog/" == feed.link[0].attrib["href"]
    assert "http://x.com/custom-blog/atom.xml" == feed.link[1].attrib["href"]
    assert "self" == feed.link[1].attrib["rel"]
    assert "Lektor Atom Plugin" == feed.generator

    assert 2 == len(feed.entry)
    post2, post1 = feed.entry  # Most recent first.

    assert "Post 2" == post2.title
    assert "2015-12-13T00:00:00Z" == post2.updated
    assert "<p>bar</p>" == str(post2.content).strip()
    assert "html" == post2.content.attrib["type"]
    assert "http://x.com/custom-blog/post2/" == post2.link.attrib["href"]
    base = post2.attrib["{http://www.w3.org/XML/1998/namespace}base"]
    assert "http://x.com/custom-blog/post2/" == base
    assert "Armin Ronacher" == post2.author.name

    assert "Post 1" == post1.title
    assert "2015-12-12T12:34:56Z" == post1.updated
    assert "<p>foo</p>" == str(post1.content).strip()
    assert "html" == post1.content.attrib["type"]
    assert "http://x.com/custom-blog/post1/" == post1.link.attrib["href"]
    base = post1.attrib["{http://www.w3.org/XML/1998/namespace}base"]
    assert "http://x.com/custom-blog/post1/" == base
    assert "A. Jesse Jiryu Davis" == post1.author.name


def test_virtual_resolver(pad, builder):
    # Pass a virtual source path to url_to().
    feed_path = "/typical-blog@atom/feed-one"
    url_path = pad.get("typical-blog/post1").url_to(feed_path)
    assert url_path == "../../typical-blog/feed.xml"

    # Pass the AtomFeedSource instance itself to url_to().
    feed_instance = pad.get(feed_path)
    assert feed_instance and feed_instance.feed_name == "Feed One"
    url_path = pad.get("typical-blog/post1").url_to(feed_instance)
    assert url_path == "../../typical-blog/feed.xml"

    feed_instance = pad.get("typical-blog2@atom/feed-two")
    assert feed_instance and feed_instance.feed_name == "feed-two"

    feed_instance = pad.get("custom-blog@atom/feed-three")
    assert feed_instance and feed_instance.feed_name == "Feed Three"
    url_path = pad.get("custom-blog/post1").url_to(feed_instance)
    assert url_path == "../../custom-blog/atom.xml"


def test_dependencies(pad, builder, reporter):
    reporter.clear()
    builder.build(pad.get("typical-blog@atom/feed-one"))

    assert set(reporter.get_recorded_dependencies()) == set(
        [
            "Website.lektorproject",
            "content/typical-blog",
            "content/typical-blog/contents.lr",
            "content/typical-blog/post1/contents.lr",
            "content/typical-blog/post2/contents.lr",
            "models/blog.ini",
            "models/blog-post.ini",
            "configs/atom.ini",
        ]
    )



# -*- coding: utf-8 -*-
import hashlib
import sys
import uuid
from datetime import date
from datetime import datetime

import click
import pkg_resources
from lektor.build_programs import BuildProgram
from lektor.context import get_ctx
from lektor.context import url_to
from lektor.db import F
from lektor.environment import Expression
from lektor.pluginsystem import Plugin
from lektor.sourceobj import VirtualSourceObject
from lektor.utils import build_url
from markupsafe import escape
from werkzeug.contrib.atom import AtomFeed

PY2 = sys.version_info[0] == 2

if PY2:
    text_type = unicode
else:
    text_type = str


class AtomFeedSource(VirtualSourceObject):
    def __init__(self, parent, feed_id, plugin):
        VirtualSourceObject.__init__(self, parent)
        self.plugin = plugin
        self.feed_id = feed_id

    @property
    def path(self):
        return self.parent.path + "@atom/" + self.feed_id

    @property
    def url_path(self):
        p = self.plugin.get_atom_config(self.feed_id, "url_path")
        if p:
            return p

        return build_url([self.parent.url_path, self.filename])

    def __getattr__(self, item):
        try:
            return self.plugin.get_atom_config(self.feed_id, item)
        except KeyError:
            raise AttributeError(item)

    @property
    def feed_name(self):
        return self.plugin.get_atom_config(self.feed_id, "name") or self.feed_id


def get(item, field, default=None):
    if field in item:
        return item[field]
    return default


def get_id(s):
    b = hashlib.md5(s.encode("utf-8")).digest()
    return uuid.UUID(bytes=b, version=3).urn


def get_item_title(item, field):
    if field in item:
        return item[field]
    return item.record_label


def get_item_body(item, field):
    if field not in item:
        raise RuntimeError("Body field %r not found in %r" % (field, item))
    with get_ctx().changed_base_url(item.url_path):
        return text_type(escape(item[field]))


def get_item_updated(item, field):
    if field in item:
        rv = item[field]
    else:
        rv = datetime.utcnow()
    if isinstance(rv, date) and not isinstance(rv, datetime):
        rv = datetime(*rv.timetuple()[:3])
    return rv


class AtomFeedBuilderProgram(BuildProgram):
    def produce_artifacts(self):
        self.declare_artifact(
            self.source.url_path, sources=list(self.source.iter_source_filenames())
        )

    def build_artifact(self, artifact):
        ctx = get_ctx()
        feed_source = self.source
        blog = feed_source.parent

        summary = get(blog, feed_source.blog_summary_field) or ""
        if hasattr(summary, "__html__"):
            subtitle_type = "html"
            summary = text_type(summary.__html__())
        else:
            subtitle_type = "text"
        blog_author = text_type(get(blog, feed_source.blog_author_field) or "")
        generator = (
            "Lektor Atom Plugin",
            "https://github.com/ajdavis/lektor-atom",
            pkg_resources.get_distribution("lektor-atom").version,
        )

        feed = AtomFeed(
            title=feed_source.feed_name,
            subtitle=summary,
            subtitle_type=subtitle_type,
            author=blog_author,
            feed_url=url_to(feed_source, external=True),
            url=url_to(blog, external=True),
            id=get_id(ctx.env.project.id),
            generator=generator,
        )

        if feed_source.items:
            # "feed_source.items" is a string like "site.query('/blog')".
            expr = Expression(ctx.env, feed_source.items)
            items = expr.evaluate(ctx.pad)
        else:
            items = blog.children

        if feed_source.item_model:
            items = items.filter(F._model == feed_source.item_model)

        order_by = "-" + feed_source.item_date_field
        items = items.order_by(order_by).limit(int(feed_source.limit))

        for item in items:
            try:
                item_author_field = feed_source.item_author_field
                item_author = get(item, item_author_field) or blog_author

                feed.add(
                    get_item_title(item, feed_source.item_title_field),
                    get_item_body(item, feed_source.item_body_field),
                    xml_base=url_to(item, external=True),
                    url=url_to(item, external=True),
                    content_type="html",
                    id=get_id(
                        u"%s/%s" % (ctx.env.project.id, item["_path"].encode("utf-8"))
                    ),
                    author=item_author,
                    updated=get_item_updated(item, feed_source.item_date_field),
                )
            except Exception as exc:
                msg = "%s: %s" % (item["_id"], exc)
                click.echo(click.style("E", fg="red") + " " + msg)

        with artifact.open("wb") as f:
            f.write(feed.to_string().encode("utf-8"))


class AtomPlugin(Plugin):
    name = u"Lektor Atom plugin"
    description = u"Lektor plugin that generates Atom feeds."

    defaults = {
        "source_path": "/",
        "name": None,
        "url_path": None,
        "filename": "feed.xml",
        "blog_author_field": "author",
        "blog_summary_field": "summary",
        "items": None,
        "limit": 50,
        "item_title_field": "title",
        "item_body_field": "body",
        "item_author_field": "author",
        "item_date_field": "pub_date",
        "item_model": None,
    }

    def get_atom_config(self, feed_id, key):
        default_value = self.defaults[key]
        return self.get_config().get("%s.%s" % (feed_id, key), default_value)

    def on_setup_env(self, **extra):
        self.env.add_build_program(AtomFeedSource, AtomFeedBuilderProgram)

        @self.env.virtualpathresolver("atom")
        def feed_path_resolver(node, pieces):
            if len(pieces) != 1:
                return

            _id = pieces[0]

            config = self.get_config()
            if _id not in config.sections():
                return

            source_path = self.get_atom_config(_id, "source_path")
            if node.path == source_path:
                return AtomFeedSource(node, _id, plugin=self)

        @self.env.generator
        def generate_feeds(source):
            for _id in self.get_config().sections():
                if source.path == self.get_atom_config(_id, "source_path"):
                    yield AtomFeedSource(source, _id, self)



import ast
import io
import re

from setuptools import setup

with io.open("README.md", "rt", encoding="utf8") as f:
    readme = f.read()

_description_re = re.compile(r"description\s+=\s+(?P<description>.*)")

with open("lektor_atom.py", "rb") as f:
    description = str(
        ast.literal_eval(_description_re.search(f.read().decode("utf-8")).group(1))
    )

tests_require = (["lxml", "pytest"],)


setup(
    author=u"A. Jesse Jiryu Davis",
    author_email="jesse@emptysquare.net",
    description=description,
    install_requires=[
        "MarkupSafe",
        "Werkzeug<1.0",  # Werkzeug 1.0 removed the feed generator
    ],
    keywords="Lektor plugin static-site blog atom rss",
    license="MIT",
    long_description=readme,
    long_description_content_type="text/markdown",
    name="lektor-atom",
    py_modules=["lektor_atom"],
    url="https://github.com/nixjdm/lektor-atom",
    version="0.3.1",
    classifiers=[
        "Environment :: Plugins",
        "Environment :: Web Environment",
        "Framework :: Lektor",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
    ],
    entry_points={"lektor.plugins": ["atom = lektor_atom:AtomPlugin"]},
    extras_require={"test": tests_require},
    tests_require=tests_require,
)
